%% Robust Dynamic Energy Use and Climate Change (Coauthors: Xin Li, Borghan Narajabad, and Ted Temzelides)
% -- Normal Distribution of Gamma with unlimited oil and coal --
% For a detail description of the numerical method, please see Appendix 6.2.1 of the paper.

%% define economic parameters
global Ndeg_1 Nnode_1 z_1 Ndeg_2 Nnode_2 z_2 Ndeg_3 Nnode_3 z_3 T3 TT3 dT3 ddT3 weights3
global dz_1 node_1 amax_1 amin_1 ext_1 extamin_1 extamax_1 dz_2 node_2 amax_2 amin_2 ext_2 extamin_2 extamax_2
global dz_3 node_3 amax_3 amin_3 ext_3 extamin_3 extamax_3
global beta phi_L phi_0 phi theta nu alpha A_bar Delta Pmax Tmax Nmax A2 A3 kappa1 kappa2 kappa3 rho g mu sigma2

% define economic parameters
beta=0.985^10; %Nordhaus: 0.985^10, Stern: 0.999^10
phi_L=0.2; % Golosov
phi_0=0.392; % Golosov
phi=0.0228; % Golosov
theta=0.3; % capital share, Golosov
nu=0.04; % energy share, Golosov
% over a fairly short period of time, Golosov
A2=7693; % coal sector
A3=1311; % green sector
kappa1=0.5008; % for oil sector
kappa2=0.08916; % for coal sector
kappa3=1-kappa1-kappa2;
rho=-0.058;
mu=2.379e-5/theta/beta; % the mean of gamma in the approximating model, set according to Golosov
sigma2=2.385e-9/(theta^2*beta^2); % the variance of gamma in the approximating model, set according to Golosov
g=(1+0.02)^10-1; % the growth rate of productivity enhanced by population growth. As seen in Model 1.9, it is equivalent
% to assume polulation grows at the rate g.

%% Define the switches
I_numerical=1;
I_shape=0;
I_sim=1;

% Define Value function Iteration criterion and number of period for
% simulation
maxit=500;
tol=1e-6;
N_period=21;

%% define robustness parameters
alpha=0.01; % penalty parameter, 
Pmax=3000;
Tmax=3000;
Nmax=100;
A_bar=theta/(1-beta*theta);
Delta=A_bar*sigma2/alpha/2;

%% define chebyshev nodes along the 1st dimension P, the permanent component of S.
Nnode_1=6; Ndeg_1=5;  % Ndeg<=Nnode-1 to get orthogonality.
[z_1 T_1 dT_1 ddT_1 weights_1]=Cheb_nodes(Nnode_1,Ndeg_1);

% define chebyshev nodes along the 2nd dimension T, the temporary component of S.
Nnode_2=7; Ndeg_2=6;  % Ndeg<=Nnode-1 to get orthogonality.
[z_2 T_2 dT_2 ddT_2 weights_2]=Cheb_nodes(Nnode_2,Ndeg_2);

% define chebyshev nodes along the 3rd dimension N, the population.
Nnode_3=10; Ndeg_3=9;  % Ndeg<=Nnode-1 to get orthogonality.
[z_3 T_3 dT_3 ddT_3 weights_3]=Cheb_nodes(Nnode_3,Ndeg_3);


% Define the overall T3
T3=kron(T_1,kron(T_2,T_3));
weights3=kron(weights_1,kron(weights_2,weights_3));
TT3=sparse(diag(1./weights3)); % =T3'*T3;
% dT3=[kron(dT_1,kron(T_2,T_3));kron(T_1,kron(dT_2,T_3));-kron(T_1,kron(T_2,dT_3))];
% three dimentional gradients, '-' since value function is increasing in R
% ddT3=[kron(ddT_1,kron(T_2,T_3));kron(T_1,kron(ddT_2,T_3));kron(T_1,kron(T_2,ddT_3))];
% three dimentional Hessian, for simplicity, don't calculate the cross derivatives

%% define the grid of points for each dimension (refer to Table 2 in the Appendix 6.2.1 of the paper)
% define the mapping from [-1,1] to state 1 (P)
amin_1=-200; amax_1=Pmax; % amax_1 should be smaller than 1/Delta, extend amin_1 to obtain accuracy
[node_1 ext_1 extamin_1 extamax_1 dz_1]=Mapping(amax_1,amin_1,z_1);

% define the mapping from [-1,1] to state 2 (T)
amin_2=-200; amax_2=Tmax; % amax_2 should be smaller than 1/Delta, extend amin_2 to obtain accuracy
[node_2 ext_2 extamin_2 extamax_2 dz_2]=Mapping(amax_2,amin_2,z_2);

% define the mapping from [-1,1] to state 3 (N)
amin_3=0.8; amax_3=Nmax; % 55 meets g=0.21 and N_period=20
[node_3 ext_3 extamin_3 extamax_3 dz_3]=Mapping(amax_3,amin_3,z_3);


%% define test nodes:
multi=2;
test_T_1=Cheb_test_nodes(Nnode_1,Ndeg_1,z_1,multi);
test_T_2=Cheb_test_nodes(Nnode_2,Ndeg_2,z_2,multi);
test_T_3=Cheb_test_nodes(Nnode_3,Ndeg_3,z_3,multi);
test_T=kron(test_T_1,kron(test_T_2,test_T_3));


%% main iteration
if I_numerical==1
    % load Coef.mat
    coef=zeros((Ndeg_1+1)*(Ndeg_2+1)*(Ndeg_3+1),1);
    E_opt=zeros(Nnode_1,Nnode_2,Nnode_3,3);
    test_f=test_T*coef; % the value at the standard node_1(s)
    crit=zeros(maxit,1);
    exitflag=zeros(Nnode_1,Nnode_2,Nnode_3);
    exit_quad=zeros(maxit,1);
    N_shape=0; % the NO. of shape-preserving
    time=cputime;
    
    matlabpool close force local
    matlabpool open local 7  % requested 7 nodes in the Mix_regime_par.pbs
    for iter=1:maxit
        f=zeros(Nnode_1,Nnode_2,Nnode_3); % vectorized value function
        test_f_old=test_f; % base on coef in the previous round
        parfor i=1:Nnode_1 % state 1 (P)
            [f(i,:,:) E_opt(i,:,:,:) exitflag(i,:,:)]=Bellman_mix_par(coef,i,beta,phi_L,phi_0,phi,theta,nu,A_bar,Delta,g,mu,...
                Ndeg_1,node_1,dz_1,extamin_1,Nnode_2,Ndeg_2,node_2,dz_2,extamin_2,Nnode_3,Ndeg_3,node_3,dz_3,extamin_3,...
                A2,A3,kappa1,kappa2,kappa3,rho);
        end
        f=reshape(permute(f,[3 2 1]),Nnode_1*Nnode_2*Nnode_3,1); % vectorized value function
        coef=weights3.*(T3'*f); % update coefficients
        % if the value function is not decreasing or not concave in S, then
        % perform the shape-preserving method
        if I_shape==1
            if (max(dT3*coef)>0 || max(ddT3*coef)>0)
                [coef exit_quad(iter)]=shape_pre(f,coef);
                N_shape=N_shape+1;
            end
        end
        
        test_f=test_T*coef;
        crit(iter)=max(abs(test_f-test_f_old));
        if crit(iter)<tol
            fprintf('Iter=%iter, crit=%1.3e\n',iter,crit(iter));
            break;
        end
        if mod(iter,1)==0
            fprintf('Iter=%iter, crit=%1.3e\n',iter,crit(iter));
        end
    end
    fprintf('Running Time=%f sec\n',cputime-time);
    v_fn=permute(reshape(f,Nnode_3,Nnode_2,Nnode_1),[3 2 1]);
    save('Mix_regime.mat','iter','N_shape','coef','E_opt','exitflag','v_fn');
    matlabpool close force local
end

%% Simulation
if I_sim==1
    if I_numerical==0
        load Coef.mat
    end
    N_seq=zeros(N_period+1,1);
    P_seq=zeros(N_period+1,1);
    T_seq=zeros(N_period+1,1);
    E_seq=zeros(N_period,3);
    % initial N, P, T and R
    N_seq(1)=1; % Golosov
    P_seq(1)=103; % Golosov?
    T_seq(1)=699-581; % Golosov?
    for t=1:N_period
        E_seq(t,:)=Bellman_mix_sim(coef,N_seq(t),P_seq(t),T_seq(t)); % x_opt=(E1,E2,E3)
        N_seq(t+1)=N_seq(t)*(1+g);
        P_seq(t+1)=P_seq(t)+phi_L*(E_seq(t,1)+E_seq(t,2));
        T_seq(t+1)=(1-phi)*T_seq(t)+(1-phi_L)*phi_0*(E_seq(t,1)+E_seq(t,2));
    end
    save('Simulation.mat','coef','N_period','N_seq','P_seq','T_seq','E_seq');
end











